/*
 *  Ledmátrix (Max7219)	8x8
 *	Mintakódok: https://github.com/MajicDesigns/MD_MAX72XX/tree/master/examples
 *  online editor: https://xantorohara.github.io/led-matrix-editor/
 */

//az alábbi kódot nagyrészt a
//mintakódokból ollóztuk össze

#include <MD_MAX72xx.h>
#include <SPI.h>

//Definiáljuk a kijelző pinjeit
#ifdef ESP8266
  //ESP8266/NodeMCU esetén
  #define CLK_PIN   D5 
  #define DATA_PIN  D7 
  #define CS_PIN 	D8 
#else
  //Arduino esetén
  #define CLK_PIN  10 
  #define DATA_PIN  12 
  #define CS_PIN 	11 
#endif


//HW típus beállítása, bővebb infó:
//https://github.com/MajicDesigns/MD_MAX72XX/
#define HARDWARE_TYPE MD_MAX72XX::GENERIC_HW
//Jelenleg 4 kijelzőnk van, de összefűzhető több eszköz is
#define MAX_DEVICES 4
MD_MAX72XX mx = MD_MAX72XX(HARDWARE_TYPE, CS_PIN, MAX_DEVICES);

//maximum szöveg hossz
const uint8_t MESG_SIZE = 255;
//két karakter közötti "szünet"
const uint8_t CHAR_SPACING = 1; //1 oszlop
//scroll animáció késleltetése (ms)
const uint8_t SCROLL_DELAY = 75; 

//szöveg tárolása (jelenlegi szöveg, amit épp scrolloz a kijelző)
char curMessage[MESG_SIZE];
//új szöveg tárolója, amit majd átmásolunk a jelenlegi szöveg tárolójába
//így, ha új szöveget kap a kijelző nem váltja át egyszerre, tudjuk időzíteni
char newMessage[MESG_SIZE];
//jelezzük, ha van új feldolgozandó üzenet
bool newMessageAvailable = false;

//callback funkció, amit az MX_MAX72XX lib hív folyamatosan 
//a funkcióban történik maga a szöveg scrollozása. a mintakódból változatlanul emeltük át
uint8_t scrollDataSource(uint8_t dev, MD_MAX72XX::transformType_t t)
{
  static enum { S_IDLE, S_NEXT_CHAR, S_SHOW_CHAR, S_SHOW_SPACE } state = S_IDLE;
  static char *p;
  static uint16_t curLen, showLen;
  static uint8_t  cBuf[8];
  uint8_t colData = 0;

  // finite state machine to control what we do on the callback
  switch (state)
  {
  case S_IDLE: // reset the message pointer and check for new message to load
    p = curMessage;      // reset the pointer to start of message
    if (newMessageAvailable)  // there is a new message waiting
    {
      strcpy(curMessage, newMessage); // copy it in
      newMessageAvailable = false;
    }
    state = S_NEXT_CHAR;
    break;

  case S_NEXT_CHAR: // Load the next character from the font table
    if (*p == '\0')
      state = S_IDLE;
    else
    {
      showLen = mx.getChar(*p++, sizeof(cBuf) / sizeof(cBuf[0]), cBuf);
      curLen = 0;
      state = S_SHOW_CHAR;
    }
    break;

  case S_SHOW_CHAR: // display the next part of the character
    colData = cBuf[curLen++];
    if (curLen < showLen)
      break;

    // set up the inter character spacing
    showLen = (*p != '\0' ? CHAR_SPACING : (MAX_DEVICES*COL_SIZE)/2);
    curLen = 0;
    state = S_SHOW_SPACE;
    // fall through

  case S_SHOW_SPACE:  // display inter-character spacing (blank column)

    curLen++;
    if (curLen == showLen)
      state = S_NEXT_CHAR;
    break;

  default:
    state = S_IDLE;
  }

  return(colData);
}

void scrollText(void) {
  static uint32_t prevTime = 0;
  // Is it time to scroll the text?
  if (millis() - prevTime >= SCROLL_DELAY)
  {
      //scroll iránya, MD_MAX72XX::TSL = jobbról balra, MD_MAX72XX::TSR = balról jobbra 
      //további lehetőségek : 
      //https://github.com/MajicDesigns/MD_MAX72XX/blob/e5bcc52ccb7b09fc8eb83bfbe114d73de8a9223d/src/MD_MAX72xx.h#L321
	  mx.transform(MD_MAX72XX::TSL); 
	  // scroll along - the callback will load all the data
	  prevTime = millis();      // starting point for next time
  }
}

void displayMessage(String msg) {
  newMessageAvailable = true;
  msg.toCharArray(newMessage,MESG_SIZE);
}

void setup() {
	Serial.begin(115200);
	Serial.println("Indulas...");
	
	mx.begin();
	mx.setShiftDataInCallback(scrollDataSource);
	//mx.setShiftDataOutCallback(scrollDataSink);
	
	//char tombok nullazasa
	curMessage[0] = newMessage[0] = '\0';
	
	//alapertelmezett szoveg indulasnal
	displayMessage("TechFactory.hu");
}

void loop() {
  //uj szoveg fogadása a konzolból
  if (Serial.available()>0) {
      String packet = Serial.readStringUntil('\n');
      //packet.remove(packet.length()-1,1);
      displayMessage(packet);
      Serial.println("Ok");
  }  
  
  //szoveg scroll
  scrollText();
}
