/*
 *  Futófény NeoPixel (WS2812b/Ws2811) címezhető ledszalaggal
 */
 
//A NeoPixel (WS2811 / WS2812) ledszalagok vezérléséhez szükséges lib
//Vázlat menüpont -> Könyvtárak tartalmazása -> Könyvtárak kezelése menüpont, keresés Neopixel, telepítés
#include <Adafruit_NeoPixel.h>

//Definiáljuk a NeoPixel adat pinjét
#ifdef ESP8266
  //ESP8266/NodeMCU esetén
  #define NEOPIXEL_PIN 5  //GPIO5 az esp8266-on = D1 a NodeMcu boardon
#else
  //Arduino esetén
  #define NEOPIXEL_PIN 3
#endif

//beállítjuk, hogy pontosan mennyi pixelből (ledből) áll a ledszalagunk
//amit meg kell címezni
#define PIXELS_COUNT 13

//RGB adatstruktúra
struct RGB {
   byte r;  //piros led
   byte g;  //zöld led
   byte b;  //kék led
};

//globális változóként létrehozzuk a NeoPixel vezérlő class-t
Adafruit_NeoPixel pixels;

/*
 * Beállítássok
 */
 
//futófényünk 'hossza'
uint8_t leds = 1;
//animáció sebessége (ms-ben)
int anim_speed = 100;
//futófény színe
RGB anim_color = {0,182,255}; //techfactory-kék :)
//automatikus szín állítás a futófényen (rainbow color)
bool auto_color = false;

/*
 * szükséges belső változók
 * az 'animáció' végrehajtásához
 */
//futófény jelenlegi pozíciója
uint8_t light_position = 0;
//futófény animáció iránya
bool light_reversed = false; //false = 0 -> PIXELS_COUNT, true = PIXELS_COUNT -> 0
//futófény animáció időzítéséhez használjuk
unsigned long anim_time = 0;
//az automatikus színkeverő funkcióhoz szükséges
uint8_t color_num_helper = 0;

void setup() {
  //NeoPixel konfigurálása
  //ledszalag típusa szerint, mi most ws2812b-t használunk
  pixels.updateType( NEO_KHZ800 + NEO_GRB );
  pixels.setPin(NEOPIXEL_PIN);
  pixels.updateLength(PIXELS_COUNT);
  pixels.begin();
}

void loop() {
    //animáció futtatása a megadott időközönként
    if (anim_time + anim_speed < millis()) {
       
         //először állítsuk be a futófényünk színét
         //ha engedélyezve van az auto_color, egyébként a megadott színnel megy
         if (auto_color) {
            anim_color = colorWheel(color_num_helper); 
            //majd növeljük a számot, hogy a következő ciklusnál ismét más színt adjon vissza
            color_num_helper++; 
            //mivel ez egy uint8_t típus, 0-255 között tárol számokat. Ezért, ha 255 után +1-et hozzáadunk a változó 
            //átfordul és ismét 0 lesz az értéke.
         }
      
         //ellenőrizzük, hogy a futófényünk a teljes ledszalagon belül helyezkedik el
         //beleszámítva a pozíciót
         //                  0                <--         -->                   13
         //  ledszalag eleje |---------------[futofeny ledek]-------------------| ledszalag vége
         
         if (PIXELS_COUNT > light_position + leds && light_position >= 0) {
            //tudatjuk a neopixellel, hogy melyik ledeket kell bekapcsolni
            //azaz, 'kirajzoljuk' a futófény aktuális állapotát
            //végig lépkedünk a ledeken és megnézzük, melyiknek kell világítania, és melyiknek nem

            //kiszámoljuk, hol kellene most állnia a futófényünknek
            //első pixel
            uint8_t light_start = light_position;
            //utolsó pixel
            uint8_t light_end   = light_position+leds;
                
            for (int i=0; i<PIXELS_COUNT; i++) {
                //ha a jelenlegi led indexe a kezdő és a záró értékünk között helyezkedik el, akkor bizony
                //világítania kell
                if (i >= light_start && i < light_end) {
                    //megszínezzük a ledszalag adott pixelét 
                    pixels.setPixelColor(i, anim_color.r, anim_color.g, anim_color.b);
                
                //ez a pixel nem része a futófényünknek ezért kikapcsoljuk
                //különben megőrizné az előző állapotát
                }else {
                    //kikapcsoljuk a ledszalag pixelét
                    pixels.setPixelColor(i, 0, 0, 0);
                }
            }
            
            //adjunk hozzá, vagy vonjunk ki a light_position -ból az iránynak megfelelően
            //ezzel elérjük, hogy a következő ciklus futásakor a futófényünk elmozdul abba az irányba
            //ha növekvő sorrenben vagyunk (balról jobbra halad a futófény)
            
            if (!light_reversed) {
                //ellenőrizzük, hogy a követkető lépés még tartományon belül lesz e
                if ( light_position +  leds + 1 <  PIXELS_COUNT){
                    light_position++;
                //ha nem, akkor megfordítjuk az irányt, így a következő ciklusban megfordul a fény iránya
                }else {
                    light_reversed = true;
                    light_position--;
                }
            }
            //egyébként pedig jobbról balra
            else {
                //ellenőrizzük, hogy a követkető lépés még tartományon belül lesz e
                if ( light_position-1 >= 0){
                    light_position--;
                //ha nem, akkor megfordítjuk az irányt, így a következő ciklusban megfordul a fény iránya
                }else {
                    light_reversed = false;
                    light_position++;
                }
            }
         }
         
         //frissítjük az utolsó lépés idejét
         anim_time = millis(); 
         //tudatjuk a vezérlővel, hogy változtak a színek, küldje ki újra a ledszalagnak
         pixels.show();
    }
    //millis átfordulás figyelése
	//ha ez megtörténik, akkor az arduino/esp már legalább 49 napja folyamatosan futott; gratulálok :)
	if (millis() < anim_time) { anim_time = 0;}
}

//A funkció az Adafruit Neopixel Library példafájlokból való
//0-255 közötti számot kér, erre visszaad egy színt
//a színek sorrendje piros - zöld - kék, majd ismét piros
RGB colorWheel(uint8_t pos) {
  pos = 255-pos;
  if (pos < 85) {
    return RGB{255 - pos * 3,0,pos*3};
  }
  else if(pos < 170) {
    return RGB{0,pos * 3,255 - pos * 3};
  }
  else  {
    pos -= 170;
    return RGB{pos * 3, 255 - pos * 3, 0};
  }
}
